from .view.manager import manage as view_tc_manage
from .add.manager import manage as add_tc_manage
from .edit.manager import manage as edit_tc_manage
from .delete.manager import manage as delete_tc_manage



import click
import sys

from ....utils.folder_manager import (
    exit_if_invalid_extension,
    exit_if_invalid_task,
    Task,
)


def manage(filename, base_folder, tc_nos, editor):
    file_path = base_folder / filename
    extension = file_path.suffix[1:]
    filename_without_extension = file_path.stem

    exit_if_invalid_extension(extension)

    task = Task(base_folder, filename_without_extension, extension)

    exit_if_invalid_task(task)

    if tc_nos[0] == 0:
        tc_nos = task.get_tc_list()

    if len(tc_nos) == 0:
        click.secho("No testcases found!", fg="cyan")
    else:
        testcases = []
        tcs_data = task.get_tcs(tc_nos)
        for tc_number, tc_data in zip(tc_nos, tcs_data):
            if tc_data is None:
                click.secho(f"TC #{tc_number} does not exist\n", fg="red")
            else:
                try:
                    testcase = {}
                    click.secho(
                        f"Waiting for input file for #{tc_number} to close", fg="cyan"
                    )
                    testcase["input"] = click.edit(
                        tc_data["input"],
                        editor=editor,
                    )

                    if testcase["input"] is None:
                        testcase["input"] = tc_data["input"]

                    click.secho(testcase["input"] + "\n")

                    click.secho(
                        f"Waiting for output file for #{tc_number} to close", fg="cyan"
                    )
                    testcase["output"] = click.edit(
                        tc_data["output"],
                        editor=editor,
                    )
                    if testcase["output"] is None:
                        testcase["output"] = tc_data["output"]

                    click.secho(testcase["output"] + "\n")

                    testcases.append(testcase)

                except click.UsageError:
                    click.secho(
                        "Unable to open editor for testcase input (check config file for potential errors).\n",
                        fg="red",
                        err=True,
                    )
                    sys.exit(1)

        task.edit_tcs(tc_nos, testcases)
        click.secho("Edited testcase files", fg="green")



import sys

import click

from ...utils.folder_manager import Task
from ..run.manager import compile_source_code, is_interpreted


def manage(filename, base_folder, config_data):
    source_code_path = base_folder / filename
    extension = source_code_path.suffix[1:]
    filename_without_extension = source_code_path.stem
    if extension not in ["py", "java", "cpp"]:
        click.secho(
            f"Language {extension} is not supported (python(py), java, c++(cpp) are only supported)",
            err=True,
            fg="red",
        )
        sys.exit(1)
    elif is_interpreted[extension]:
        click.secho(
            f"Language {extension} does not support compilation", err=True, fg="red"
        )
        sys.exit(1)
    else:
        task = Task(base_folder, filename_without_extension, extension)
        task_status = task.task_exists()
        if task_status not in [1, 3]:
            click.secho("Source Code does not exist", err=True, fg="red")
            sys.exit(1)
        else:
            if task_status == 1:
                task.create_task(create_source_code=False)
            compilation_error_path = task.last_run_folder / "compilation_error.txt"
            compile_source_code(
                source_code_path, compilation_error_path, extension, config_data
            )



import sys
from pathlib import Path

import click

from ...utils.file_manager import open_source_code_in_editor
from ...utils.folder_manager import Task


def manage(filename, base_folder, force, config_data):
    file_path = base_folder / filename
    extension = file_path.suffix[1:]
    filename_without_extension = file_path.stem
    if extension not in ["py", "java", "cpp"]:
        click.secho(
            f"Language {extension} is not supported (python(py), java, c++(cpp) are only supported)",
            err=True,
            fg="red",
        )
        sys.exit(1)
    else:
        task = Task(base_folder, filename_without_extension, extension)
        task.safe_overwrite(force)
        task.create_task(template_path=config_data["language"][extension]["template"])
        click.secho(f"Successfully created task", fg="green")
        click.secho(
            f"Source code is located at {task.source_code.resolve()}", fg="green"
        )
        open_source_code_in_editor(config_data, task.source_code)



import subprocess
import click
import sys
from pathlib import Path

from ...utils.file_manager import check_diff, is_file_empty, print_file
from ...utils.folder_manager import Task, delete_folder, clear_folder
from ..run.manager import is_interpreted, run_func, compile_source_code


def judge(
    task,
    base_folder,
    filename_without_extension,
    extension,
    gen_path,
    gen_extension,
    number_of_runs,
    config_data,
):
    clear_folder(task.last_run_folder)

    source_code_path = (base_folder / filename_without_extension).with_suffix(
        "." + extension
    )
    gen_folder = task.last_run_folder / "gen"
    gen_folder.mkdir()

    compilation_error_path = gen_folder / "compilation_error.txt"
    gen_compilation_error_path = gen_folder / "gen_compilation_error.txt"

    exec_path = Path()
    if not is_interpreted[extension]:
        exec_path = compile_source_code(
            source_code_path, compilation_error_path, extension, config_data
        )
    else:
        click.secho(f"Running the source code with command:", fg="cyan")
        click.secho(config_data["language"][extension]["command"] + "\n")

    gen_exec_path = Path()
    if not is_interpreted[gen_extension]:
        gen_exec_path = compile_source_code(
            gen_path, gen_compilation_error_path, gen_extension, config_data, True
        )
    else:
        click.secho(f"Running the generator code with command:", fg="cyan")
        click.secho(config_data["language"][gen_extension]["command"] + "\n")

    for current_run in range(number_of_runs):
        # gen input (empty file)
        gen_in_path = gen_folder / "gen_in.txt"
        gen_in_path.touch()

        # stdout of gen is same as the generated tc
        gen_std_output_path = gen_folder / "tc.txt"
        gen_std_error_path = gen_folder / "gen_error.txt"

        gen_run_returncode = 0
        if is_interpreted[gen_extension]:
            gen_run_returncode = run_func[gen_extension](
                gen_path,
                config_data["language"][gen_extension]["command"],
                gen_in_path,
                gen_std_output_path,
                gen_std_error_path,
            )
        else:
            gen_run_returncode = run_func[gen_extension](
                gen_exec_path, gen_in_path, gen_std_output_path, gen_std_error_path
            )

        if gen_run_returncode != 0:
            click.secho(f"Rumtime Error while generating testcase", fg="red")
            print_file(gen_std_error_path, 2)
            sys.exit(1)

        std_output_path = gen_folder / f"output.txt"
        std_error_path = gen_folder / f"error.txt"

        run_returncode = 0
        if is_interpreted[extension]:
            run_returncode = run_func[extension](
                source_code_path,
                config_data["language"][extension]["command"],
                gen_std_output_path,
                std_output_path,
                std_error_path,
            )
        else:
            run_returncode = run_func[extension](
                exec_path, gen_std_output_path, std_output_path, std_error_path
            )

        if run_returncode != 0:
            click.secho(f"Rumtime Error while running source code\n", fg="red")
            print_file(std_error_path, 2)
            click.secho(f"Testcase: ", fg="cyan")
            print_file(gen_std_output_path)
            sys.exit(0)

        if is_file_empty(std_output_path):
            click.secho(f"Accepted #{current_run + 1}\n", fg="green")

            if not is_file_empty(std_error_path):
                click.secho(f"Standard Error: ", fg="cyan")
                print_file(std_error_path)
        else:
            click.secho(f"Wrong Answer #{current_run + 1}\n", fg="red")

            click.secho(f"Test Case: ", fg="cyan")
            print_file(gen_std_output_path)

            click.secho(f"Standard Output: ", fg="cyan")
            print_file(std_output_path)

            if not is_file_empty(std_error_path):
                click.secho(f"Standard Error: ", fg="cyan")
                print_file(std_error_path)
            sys.exit(1)

    click.secho(f"Successfully passed {number_of_runs} testcases!", fg="green")


def manage(filename, number_of_runs, gen_path, base_folder, config_data):
    file_path = base_folder / filename
    extension = file_path.suffix[1:]

    gen_extension = gen_path.suffix[1:]

    filename_without_extension = file_path.stem
    if extension not in ["py", "java", "cpp"] or gen_extension not in [
        "py",
        "java",
        "cpp",
    ]:
        click.secho(
            f"Language {extension} is not supported (python(py), java, c++(cpp) are only supported)",
            err=True,
            fg="red",
        )
        sys.exit(1)
    else:
        task = Task(base_folder, filename_without_extension, extension)
        task_status = task.task_exists()
        if task_status not in [1, 3]:
            click.secho("Source Code does not exist", err=True, fg="red")
            sys.exit(1)
        else:
            if task_status == 1:
                task.create_task(create_source_code=False)
            judge(
                task,
                base_folder,
                filename_without_extension,
                extension,
                gen_path,
                gen_extension,
                number_of_runs,
                config_data,
            )



import click

from ....utils.folder_manager import (
    exit_if_invalid_extension,
    exit_if_invalid_task,
    Task,
)


def manage(filename, base_folder, tcs):
    file_path = base_folder / filename
    extension = file_path.suffix[1:]
    filename_without_extension = file_path.stem

    exit_if_invalid_extension(extension)

    task = Task(base_folder, filename_without_extension, extension)

    exit_if_invalid_task(task)

    if tcs[0] == 0:
        tcs = task.get_tc_list()

    if len(tcs) == 0:
        click.secho("No testcases found!", fg="cyan")
    else:
        tcs_data = task.get_tcs(tcs)
        for index, tc_data in enumerate(tcs_data):
            if tc_data is None:
                click.secho(f"TC #{tcs[index]} does not exist\n", fg="red")
            else:
                click.secho(f"Input #{tcs[index]}:", fg="cyan")
                click.secho(tc_data["input"] + "\n")

                click.secho(f"Output #{tcs[index]}: ", fg="cyan")
                click.secho(tc_data["output"] + "\n")



from pathlib import Path

import click
import sys

from ..commands import (
    compile_manage,
    config_manage,
    create_manage,
    fetch_manage,
    run_manage,
    test_manage,
    view_tc_manage,
    add_tc_manage,
    edit_tc_manage,
    delete_tc_manage,
)

from ..utils.config_manager import get_config_data

CONTEXT_SETTINGS = dict(help_option_names=["-h", "--help"])


@click.group(context_settings=CONTEXT_SETTINGS)
def cli():
    global config_data
    config_data = get_config_data()


@cli.command("create", short_help="create a task")
@click.argument("filename", type=str)
@click.option(
    "-p",
    "--path",
    "base_folder",
    default=None,
    type=click.Path(exists=True, path_type=Path, writable=True, file_okay=False),
    help="path to the folder where you want to create the source code file",
)
@click.option("-f", "--force", is_flag=True, help="overwrite existing files of task")
def create(filename, base_folder, force):
    """
    \b
    Create a task to add and run testcases

    \b
    Args:

    \b
    FILENAME of the file to be created with file extension
    """
    if base_folder is None:
        base_folder = Path(config_data["default_base_folder"])
    create_manage(filename, base_folder, force, config_data)


@cli.command("fetch", short_help="retrieve testcase data from online judge")
@click.option(
    "-p",
    "--path",
    "base_folder",
    default=None,
    type=click.Path(exists=True, path_type=Path, writable=True, file_okay=False),
    help="path to the folder where you want to create the source code file",
)
@click.option("-f", "--force", is_flag=True, help="overwrite existing files of task")
def fetch(base_folder, force):
    """
    \b
    Retrieve testcase data from online judge,
    make the source code file,
    and copy boiler plate code if specified in config file
    """
    if base_folder is None:
        base_folder = Path(config_data["default_base_folder"])
    fetch_manage(base_folder, force, config_data)


@cli.command("run", short_help="run code against testcases")
@click.argument("filename", type=str)
@click.option(
    "-p",
    "--path",
    "base_folder",
    default=None,
    type=click.Path(exists=True, path_type=Path, writable=True, file_okay=False),
    help="path to the folder which contains the souce code",
)
@click.option(
    "-t",
    "--tc",
    "tc_no",
    default=0,
    show_default=True,
    help="run specific testcase (0 for all)",
)
@click.option(
    "-i",
    "--interactive",
    is_flag=True,
    help="run program in interactive mode(stdin, stdout, stderr are used)",
)
def run(filename, base_folder, tc_no, interactive):
    """
    \b
    Compile (if applied) and run source code on saved testcases
    using commands specified in the config file

    \b
    Args:

    \b
    FILENAME of the file to run with file extension
    """
    if base_folder is None:
        base_folder = Path(config_data["default_base_folder"])
    run_manage(filename, base_folder, config_data, tc_no, interactive)


@cli.command("compile", short_help="compile source code")
@click.argument("filename", type=str)
@click.option(
    "-p",
    "--path",
    "base_folder",
    default=None,
    type=click.Path(exists=True, path_type=Path, writable=True, file_okay=False),
    help="path to the folder which contains the souce code",
)
def compile(filename, base_folder):
    """
    \b
    Compile (if applied) the source code
    using commands specified in the config file

    \b
    Args:

    \b
    FILENAME of the file to run with file extension
    """
    if base_folder is None:
        base_folder = Path(config_data["default_base_folder"])
    compile_manage(filename, base_folder, config_data)


@cli.group(short_help="commands related to testcase data")
def tc():
    pass


def get_cleaned_tcs(tcs):
    cleaned_tcs = []

    has_zero = len(tcs) == 0
    for currentTc in tcs:
        if int(currentTc) == 0:
            has_zero = True
        else:
            cleaned_tcs.append(int(currentTc))

    if has_zero:
        cleaned_tcs = [0]

    cleaned_tcs = list(set(cleaned_tcs))
    cleaned_tcs.sort()
    return cleaned_tcs


@tc.command(short_help="view testcases")
@click.argument("filename", type=str)
@click.option(
    "-p",
    "--path",
    "base_folder",
    default=None,
    type=click.Path(
        exists=True,
        path_type=Path,
        writable=True,
        file_okay=False,
    ),
    help="path to the folder which contains the souce code",
)
@click.argument("tcs", nargs=-1)
def view(filename, base_folder, tcs):
    """
    \b
    View a set of testcases related to FILENAME

    \b
    Args:

    \b
    FILENAME of the source code file with file extension
    TCS: space seperated list of test case numbers (0 for all)
    """
    if base_folder is None:
        base_folder = Path(config_data["default_base_folder"])

    view_tc_manage(filename, base_folder, get_cleaned_tcs(tcs))


@tc.command(short_help="add testcase")
@click.argument("filename", type=str)
@click.option(
    "-p",
    "--path",
    "base_folder",
    default=None,
    type=click.Path(
        exists=True,
        path_type=Path,
        writable=True,
        file_okay=False,
    ),
    help="path to the folder which contains the souce code",
)
def add(filename, base_folder):
    """
    \b
    Add a testcase to FILENAME

    \b
    Args:

    \b
    FILENAME of the source code file with file extension
    """
    if base_folder is None:
        base_folder = Path(config_data["default_base_folder"])

    editor = None
    if "multiline_input_command" in config_data:
        editor = config_data["multiline_input_command"]

    add_tc_manage(filename, base_folder, editor)


@tc.command(short_help="edit testcases")
@click.argument("filename", type=str)
@click.option(
    "-p",
    "--path",
    "base_folder",
    default=None,
    type=click.Path(
        exists=True,
        path_type=Path,
        writable=True,
        file_okay=False,
    ),
    help="path to the folder which contains the souce code",
)
@click.argument("tcs", nargs=-1)
def edit(filename, base_folder, tcs):
    """
    \b
    Edit a set of testcases related to FILENAME

    \b
    Args:

    \b
    FILENAME of the source code file with file extension
    TCS: space seperated list of test case numbers (0 for all)
    """
    if base_folder is None:
        base_folder = Path(config_data["default_base_folder"])

    editor = None
    if "multiline_input_command" in config_data:
        editor = config_data["multiline_input_command"]

    edit_tc_manage(filename, base_folder, get_cleaned_tcs(tcs), editor)


@tc.command(short_help="delete testcases")
@click.argument("filename", type=str)
@click.option(
    "-p",
    "--path",
    "base_folder",
    default=None,
    type=click.Path(
        exists=True,
        path_type=Path,
        writable=True,
        file_okay=False,
    ),
    help="path to the folder which contains the souce code",
)
@click.argument("tcs", nargs=-1, required=True)
def delete(filename, base_folder, tcs):
    """
    \b
    Delete a set of testcases related to FILENAME

    \b
    Args:

    \b
    FILENAME of the source code file with file extension
    TCS: space seperated list of test case numbers (0 for all)
    """
    if base_folder is None:
        base_folder = Path(config_data["default_base_folder"])

    delete_tc_manage(filename, base_folder, get_cleaned_tcs(tcs))


@cli.command("test", short_help="brute force testing")
@click.argument("filename", type=str)
@click.argument(
    "testcase-generator-path",
    type=click.Path(exists=True, path_type=Path, dir_okay=False),
)
@click.option(
    "-n",
    "--number-of-runs",
    default=10000,
    show_default=True,
    help="no of randomly generated testcases",
)
@click.option(
    "-p",
    "--path",
    "base_folder",
    default=None,
    type=click.Path(
        exists=True,
        path_type=Path,
        writable=True,
        file_okay=False,
    ),
    help="path to the folder which contains the souce code",
)
def test(filename, number_of_runs, testcase_generator_path, base_folder):
    """
    \b
    Generate random testcases and run your code against them.
    Create a function in your source code which validates
    your answer if the validation fails print your answer to
    std output.
    \b
    Args:

    \b
    FILENAME of the source code file with file extension
    TESTCASE_GENERATOR_PATH: path of test case generator file
    """
    if base_folder is None:
        base_folder = Path(config_data["default_base_folder"])
    test_manage(
        filename, number_of_runs, testcase_generator_path, base_folder, config_data
    )


@cli.command("config", short_help="get location of config file or reset config file")
@click.option("--reset", "-r", is_flag=True, help="reset config file to default values")
def config(reset):
    """
    \b
    Locate config file or reset config to default values with --reset option
    """
    config_manage(reset)



import click

from ....utils.folder_manager import (
    exit_if_invalid_extension,
    exit_if_invalid_task,
    Task,
)


def manage(filename, base_folder, delete_tc_nos):
    file_path = base_folder / filename
    extension = file_path.suffix[1:]
    filename_without_extension = file_path.stem

    exit_if_invalid_extension(extension)

    task = Task(base_folder, filename_without_extension, extension)

    exit_if_invalid_task(task)

    if delete_tc_nos[0] == 0:
        delete_tc_nos = task.get_tc_list()

    if len(delete_tc_nos) == 0:
        click.secho("No testcases found!", fg="cyan")
    else:
        tc_nos = task.get_tc_list()
        tc_data = task.get_tcs(tc_nos)
        task.clear_tcs()

        delete_tc_nos = sorted(delete_tc_nos, reverse=True)
        for delete_tc_no in delete_tc_nos:
            if delete_tc_no not in tc_nos:
                click.secho(f"TC #{delete_tc_no} does not exist\n", fg="red")
            else:
                del tc_data[delete_tc_no - 1]

        task.add_tests(tc_data)
        click.secho("Deleted testcase files", fg="green")



import click
import sys

from ....utils.folder_manager import (
    exit_if_invalid_extension,
    exit_if_invalid_task,
    Task,
)


def manage(filename, base_folder, editor):
    file_path = base_folder / filename
    extension = file_path.suffix[1:]
    filename_without_extension = file_path.stem

    exit_if_invalid_extension(extension)

    task = Task(base_folder, filename_without_extension, extension)

    exit_if_invalid_task(task)

    try:
        testcase = {}

        click.secho("Waiting for input file to close", fg="cyan")
        testcase["input"] = click.edit(
            "Enter the input and save the file (delete the help text before saving).\nYou can set the input editor in config file (the default editor is vim, use :q to exit if you opened it by mistake)",
            editor=editor,
        )
        if testcase["input"] is None:
            click.secho("Input file closed without saving\n", fg="red", err=True)
            sys.exit(1)

        click.secho(testcase["input"] + "\n")

        click.secho("Waiting for output file to close", fg="cyan")
        testcase["output"] = click.edit(
            "Enter the output and save the file (delete the help text before saving).\nYou can set the input editor in config file (the default editor is vim, use :q to exit if you opened it by mistake)",
            editor=editor,
        )
        if testcase["output"] is None:
            click.secho("Output file closed without saving\n", fg="red", err=True)
            sys.exit(1)

        click.secho(testcase["output"] + "\n")

        task.add_test(testcase)
        click.secho("Created testcase file", fg="green")
    except click.UsageError:
        click.secho(
            "Unable to open editor for testcase input (check config file for potential errors).\n",
            fg="red",
            err=True,
        )
        sys.exit(1)



import click

from ...utils.config_manager import create_config, get_config_path


def manage(reset):
    if reset:
        click.secho("Resetting config file to default version", fg="cyan")
        create_config()
    else:
        click.secho(f"Config file is located at {get_config_path()}", fg="cyan")



import click

from ...utils.file_manager import open_source_code_in_editor
from ...utils.folder_manager import Task
from .server import get_task_data


def get_file_name(task_name):
    name_char_list = []
    last_was_alnum = 0
    for ch in task_name:
        if ch.isalnum():
            name_char_list.append(ch)
            last_was_alnum = 1
        else:
            if last_was_alnum:
                name_char_list.append("_")
            last_was_alnum = 0
    return "".join(name_char_list)


def manage(base_folder, force, config_data):
    task_data = get_task_data()
    testData = task_data["tests"]
    tests = []
    for individual_testcase in testData:
        test = {}
        is_input_file = True
        for testcase_element in individual_testcase:
            if is_input_file:
                test["input"] = individual_testcase[testcase_element]
            else:
                test["output"] = individual_testcase[testcase_element]
            is_input_file = not is_input_file
        tests.append(test)

    filename_without_extension = get_file_name(task_data["name"])
    extension = config_data["default_language"]

    task = Task(base_folder, filename_without_extension, extension)
    task.safe_overwrite(force)
    task.create_task(tests, config_data["language"][extension]["template"])
    click.secho("Successfully created task", fg="green")
    click.secho(f"Source code is located at {task.source_code.resolve()}", fg="green")
    open_source_code_in_editor(config_data, task.source_code)



import sys
from pathlib import Path
from re import template

import click
import yaml

user_config_path = Path.home() / ".config/cppt/config.yaml"


def create_config():
    default_config_path = Path(__file__).parent / "default_config.yaml"
    user_config_path.parent.mkdir(parents=True, exist_ok=True)
    user_config_path.write_bytes(default_config_path.read_bytes())


def get_config_path():
    return user_config_path.resolve()


def check_config_data(data):
    is_invalid = False
    if "default_base_folder" in data:
        data["default_base_folder"] = data["default_base_folder"].replace(
            "~", str(Path.home())
        )
        default_base_folder_path = Path(data["default_base_folder"])
        default_base_folder_path.expanduser()
        if not default_base_folder_path.exists():
            click.secho(
                f"The default base folder ({default_base_folder_path.resolve()}) specified in config file does not exist",
                err=True,
                fg="red",
            )
            is_invalid = True
        if not default_base_folder_path.is_dir():
            click.secho(
                f"The default base folder ({default_base_folder_path.resolve()}) specified in config file is not a directory",
                err=True,
                fg="red",
            )
            is_invalid = True
    else:
        click.secho(
            "The default base folder was not found in config", err=True, fg="red"
        )
        is_invalid = True

    if "default_language" in data:
        if data["default_language"] not in ["cpp", "py", "java"]:
            click.secho(
                f"Invalid default language {data['default_language']}",
                err=True,
                fg="red",
            )
            click.secho(
                "python(py), java, c++(cpp) are only supported", err=True, fg="red"
            )
            is_invalid = True
    else:
        click.secho(
            "The default language was not defined in config", err=True, fg="red"
        )
        is_invalid = True

    for lang in data["language"]:
        if data["language"][lang]["template"] is not None:
            data["language"][lang]["template"] = data["language"][lang][
                "template"
            ].replace("~", str(Path.home()))
            template_path = Path(data["language"][lang]["template"])
            if not template_path.exists():
                click.secho(
                    f"template for {lang} ({template_path.resolve()}) specified in config file does not exist",
                    err=True,
                    fg="red",
                )
                is_invalid = True
            elif template_path.is_dir():
                click.secho(
                    f"{template_path.resolve()} is a directory, should be a file",
                    err=True,
                    fg="red",
                )
                is_invalid = True
            elif template_path.suffix[1:] != lang:
                click.secho(
                    f"file extension of template ({template_path.resolve()}) doesnot match with {lang}",
                    err=True,
                    fg="red",
                )
                is_invalid = True

    if is_invalid:
        sys.exit(1)


def check_config():
    if not user_config_path.exists():
        click.secho(
            f"config not found at {user_config_path.resolve()}", err=True, fg="red"
        )
        click.secho(f"creating config at {user_config_path.resolve()}", fg="cyan")
        create_config()


def get_config_data():
    check_config()
    user_config_file = open(user_config_path, "r")
    user_config = yaml.safe_load(user_config_file)
    check_config_data(user_config)
    return user_config


if __name__ == "__main__":
    get_config_data()



import sys

import click

from .file_manager import read_from_file, write_to_file


def delete_folder(path):
    for sub in path.iterdir():
        if sub.is_dir():
            delete_folder(sub)
        else:
            sub.unlink()
    path.rmdir()


def clear_folder(path):
    for sub in path.iterdir():
        if sub.is_dir():
            delete_folder(sub)
        else:
            sub.unlink()


class Task:
    def __init__(self, base_folder, task_name, extension):
        self.source_code = base_folder / task_name
        self.source_code = self.source_code.with_suffix("." + extension)
        self.cppt_root_folder = base_folder / ".cppt"
        self.task_folder = self.cppt_root_folder / task_name
        self.tc_folder = self.task_folder / "tc"
        self.last_run_folder = self.task_folder / "last_run"

    def create_root(self):
        try:
            self.cppt_root_folder.mkdir()
            click.secho("Created cppt root folder", fg="cyan")
        except Exception:
            click.secho("Unable to create cppt root folder", fg="red", err=True)
            sys.exit(1)

    def create_task(self, tests=None, template_path=None, create_source_code=True):
        if not self.cppt_root_folder.exists():
            self.create_root()

        if create_source_code:
            try:
                self.source_code.touch()
                if template_path is not None:
                    write_to_file(self.source_code, read_from_file(template_path))
                click.secho("Created source code file", fg="cyan")
            except Exception:
                click.secho("Unable to create source code file", fg="red", err=True)
                sys.exit(1)

        try:
            self.task_folder.mkdir()
            self.tc_folder.mkdir()
            self.last_run_folder.mkdir()
            click.secho("Created task folder", fg="cyan")
        except Exception:
            click.secho("Unable to create task folder", fg="red", err=True)
            sys.exit(1)

        if tests is not None:
            self.add_tests(tests)
            click.secho("Created testcase files", fg="cyan")

    def add_tests(self, tests, tc_number=1):
        try:
            testcase_number = tc_number
            for test in tests:
                self.add_test(test, testcase_number)
                testcase_number += 1

        except Exception:
            click.secho("Unable to add testcases", fg="red", err=True)
            sys.exit(1)

    def add_test(self, test, tc_number=None):
        try:
            if tc_number is None:
                tc_list = self.get_tc_list()
                tc_number = 1
                if len(tc_list) != 0:
                    tc_number = tc_list[-1] + 1

            with open(
                self.tc_folder / f"in{tc_number}.txt",
                "w",
                encoding="utf-8",
            ) as file:
                file.writelines(test["input"])

            with open(
                self.tc_folder / f"ans{tc_number}.txt",
                "w",
                encoding="utf-8",
            ) as file:
                file.writelines(test["output"])
        except Exception:
            click.secho(f"Unable to add testcase #{tc_number}", fg="red", err=True)
            sys.exit(1)

    def task_exists(self):
        """checks if task exist

        Returns:
            0: neither source code nor test data exists
            1: source code exists
            2: test data exists
            3: both exists
        """
        result = 0
        if self.source_code.exists():
            result = 1

        if self.task_folder.exists():
            if result != 0:
                result = 3
            else:
                result = 2
        return result

    def overwrite(self):
        if self.task_folder.exists():
            try:
                delete_folder(self.task_folder)
                click.secho("Deleted the existing task folder", fg="red")
            except Exception:
                click.secho(
                    f"Unable to delete task folder at {self.task_folder.resolve()}",
                    fg="red",
                    err=True,
                )
                sys.exit(1)

        if self.source_code.exists():
            try:
                self.source_code.unlink()
                click.secho("Deleted the existing source code file", fg="red")
            except Exception:
                click.secho(
                    f"Unable to delete source code at {self.source_code.resolve()}",
                    fg="red",
                    err=True,
                )
                sys.exit(1)

    def safe_overwrite(self, force):
        task_status = self.task_exists()
        if task_status != 0:
            if not force:
                if task_status == 1:
                    click.secho(
                        f"Source code already exists at {self.source_code.resolve()}",
                        err=True,
                        fg="red",
                    )
                elif task_status == 2:
                    click.secho(
                        f"Task folder already exists at {self.task_folder.resolve()}",
                        err=True,
                        fg="red",
                    )
                else:
                    click.secho(
                        f"Both source code and task folder already exists at {self.source_code.resolve()} and {self.task_folder.resolve()} respectively",
                        err=True,
                        fg="red",
                    )

                click.secho(
                    "To overwrite existing files specify --force option",
                    err=True,
                    fg="red",
                )
                sys.exit(1)
            else:
                self.overwrite()

    def get_tc_list(self):
        tc_set = set()
        for tc in self.tc_folder.iterdir():
            tc = tc.name
            if tc.startswith("in"):
                num = int(tc[2:][:-4])
                tc_set.add(num)
            elif tc.startswith("ans"):
                num = int(tc[3:][:-4])
                tc_set.add(num)

        tc_list = []
        for tc in tc_set:
            tc_list.append(tc)

        tc_list.sort()
        return tc_list

    def get_tc(self, tc_no):
        tc_list = self.get_tc_list()

        if tc_no not in tc_list:
            return None

        return {
            "input": read_from_file(self.tc_folder / f"in{tc_no}.txt"),
            "output": read_from_file(self.tc_folder / f"ans{tc_no}.txt"),
        }

    def get_tcs(self, tc_nos):
        tc_data = []
        for tc_no in tc_nos:
            tc_data.append(self.get_tc(tc_no))
        return tc_data

    def edit_tc(self, tc_no, tc):
        with open(
            self.tc_folder / f"in{tc_no}.txt",
            "w",
            encoding="utf-8",
        ) as file:
            file.writelines(tc["input"])

        with open(
            self.tc_folder / f"ans{tc_no}.txt",
            "w",
            encoding="utf-8",
        ) as file:
            file.writelines(tc["output"])

    def edit_tcs(self, tc_nos, tcs):
        for tc_no, tc in zip(tc_nos, tcs):
            self.edit_tc(tc_no, tc)

    def clear_tcs(self):
        clear_folder(self.tc_folder)


def exit_if_invalid_extension(extension):
    if extension not in ["py", "java", "cpp"]:
        click.secho(
            f"Language {extension} is not supported (python(py), java, c++(cpp) are only supported)",
            err=True,
            fg="red",
        )
        sys.exit(1)


def exit_if_invalid_task(task):
    task_status = task.task_exists()
    if task_status != 3:
        if task_status == 0:
            click.secho(
                f"Neither source code nor task folder exists at {task.source_code.resolve()} and {task.task_folder.resolve()} respectively",
                err=True,
                fg="red",
            )
        elif task_status == 1:
            click.secho(
                f"Task folder does not exist at {task.task_folder.resolve()}",
                err=True,
                fg="red",
            )
        elif task_status == 2:
            click.secho(
                f"Source code does not exist at {task.source_code.resolve()}",
                err=True,
                fg="red",
            )

        click.secho(
            "Try overwriting existing files with create or fetch commands's --force option",
            err=True,
            fg="red",
        )
        click.secho(
            "Caution: do not forget to copy any code in source file before using --force option",
            err=True,
            fg="red",
        )
        sys.exit(1)



from .compile.manager import manage as compile_manage
from .config.manager import manage as config_manage
from .create.manager import manage as create_manage
from .fetch.manager import manage as fetch_manage
from .run.manager import manage as run_manage
from .test.manager import manage as test_manage
from .tc import add_tc_manage, view_tc_manage, edit_tc_manage, delete_tc_manage



from setuptools import setup

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setup(
    name="cppt",
    version="1.3.0",
    author="Vishal Agrawal",
    author_email="vishalagrawalva22@gmail.com",
    description="Command line tool to automate your competitive programming workflow without cluttering your workspace with testcase data",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/vishalagrawal22/CPPT",
    py_modules=["cppt.bin.cppt"],
    include_package_data=True,
    install_requires=[
        "click",
        "pyyaml",
    ],
    packages=[
        "cppt.utils",
        "cppt.commands.create",
        "cppt.commands.fetch",
        "cppt.commands.run",
        "cppt.commands.config",
        "cppt.commands.compile",
        "cppt.commands.tc",
        "cppt.commands.tc.add",
        "cppt.commands.tc.edit",
        "cppt.commands.tc.view",
        "cppt.commands.tc.delete",
        "cppt.commands.test",
        "cppt.commands",
    ],
    classifiers=[
        "Programming Language :: Python :: 3",
        "Operating System :: OS Independent",
        "License :: OSI Approved :: MIT License",
    ],
    python_requires=">=3.6",
    entry_points={
        "console_scripts": [
            "cppt = cppt.bin.cppt:cli",
        ],
    },
)



import json
from http.server import BaseHTTPRequestHandler, HTTPServer

import click


class postRequestHandler(BaseHTTPRequestHandler):
    def do_POST(self):
        global task_data
        data_length = int(self.headers["Content-Length"])
        self.data = self.rfile.read(data_length)
        click.secho("Received problem data", fg="cyan")
        task_data = json.loads(self.data)


def run_server():
    PORT = 1327
    server = HTTPServer(("localhost", PORT), postRequestHandler)
    click.secho(
        "Currently only problem parsing is supported (contest parsing is not supported)",
        fg="cyan",
    )
    click.secho(f"Listening on port {PORT}", fg="cyan")
    click.secho("Waiting for competitive companion extension to send data\n", fg="cyan")
    server.handle_request()


def get_task_data():
    global task_data
    run_server()
    return task_data


if __name__ == "__main__":
    get_task_data()
